"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterQueryBuilder = void 0;
const core_1 = require("@nestjs-query/core");
const where_builder_1 = require("./where.builder");
/**
 * @internal
 *
 * Class that will convert a Query into a `sequelize` Query Builder.
 */
class FilterQueryBuilder {
    constructor(model, whereBuilder = new where_builder_1.WhereBuilder()) {
        this.model = model;
        this.whereBuilder = whereBuilder;
    }
    /**
     * Create a `sequelize` SelectQueryBuilder with `WHERE`, `ORDER BY` and `LIMIT/OFFSET` clauses.
     *
     * @param query - the query to apply.
     */
    findOptions(query) {
        let opts = this.applyAssociationIncludes({ subQuery: false }, query.filter);
        opts = this.applyFilter(opts, query.filter);
        opts = this.applySorting(opts, query.sorting);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    countOptions(query) {
        let opts = this.applyAssociationIncludes({}, query.filter);
        opts.distinct = true;
        opts = this.applyFilter(opts, query.filter);
        return opts;
    }
    /**
     * Create a `sequelize` DeleteQueryBuilder with a WHERE clause.
     *
     * @param query - the query to apply.
     */
    destroyOptions(query) {
        let opts = {};
        opts = this.applyFilter(opts, query.filter);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    /**
     * Create a `sequelize` UpdateQueryBuilder with `WHERE` and `ORDER BY` clauses
     *
     * @param query - the query to apply.
     */
    updateOptions(query) {
        let opts = { where: {} };
        opts = this.applyFilter(opts, query.filter);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    /**
     * Applies paging to a Pageable `sequelize` query builder
     * @param qb - the `sequelize` QueryBuilder
     * @param paging - the Paging options.
     */
    applyPaging(qb, paging) {
        if (!paging) {
            return qb;
        }
        if (paging.limit !== undefined) {
            // eslint-disable-next-line no-param-reassign
            qb.limit = paging.limit;
        }
        if (paging.offset !== undefined) {
            // eslint-disable-next-line no-param-reassign
            qb.offset = paging.offset;
        }
        return qb;
    }
    /**
     * Applies the filter from a Query to a `sequelize` QueryBuilder.
     *
     * @param qb - the `sequelize` QueryBuilder.
     * @param filter - the filter.
     */
    applyFilter(filterable, filter) {
        if (!filter) {
            return filterable;
        }
        // eslint-disable-next-line no-param-reassign
        filterable.where = this.whereBuilder.build(filter, this.getReferencedRelations(filter));
        return filterable;
    }
    /**
     * Applies the ORDER BY clause to a `sequelize` QueryBuilder.
     * @param qb - the `sequelize` QueryBuilder.
     * @param sorts - an array of SortFields to create the ORDER BY clause.
     */
    applySorting(qb, sorts) {
        if (!sorts) {
            return qb;
        }
        // eslint-disable-next-line no-param-reassign
        qb.order = sorts.map(({ field, direction, nulls }) => {
            const col = `${field}`;
            const dir = [direction];
            if (nulls) {
                dir.push(nulls);
            }
            return [col, dir.join(' ')];
        });
        return qb;
    }
    applyAssociationIncludes(findOpts, filter) {
        if (!filter) {
            return findOpts;
        }
        const referencedRelations = this.getReferencedRelations(filter);
        return [...referencedRelations.values()].reduce((find, association) => {
            // eslint-disable-next-line no-param-reassign
            find.include = [...(find.include || []), { association, attributes: [] }];
            return find;
        }, findOpts);
    }
    getReferencedRelations(filter) {
        const { relationNames } = this;
        const referencedFields = core_1.getFilterFields(filter);
        const referencedRelations = referencedFields.filter((f) => relationNames.includes(f));
        return referencedRelations.reduce((map, r) => {
            return map.set(r, this.model.associations[r]);
        }, new Map());
    }
    get relationNames() {
        return Object.keys(this.model.associations || {});
    }
}
exports.FilterQueryBuilder = FilterQueryBuilder;
//# sourceMappingURL=filter-query.builder.js.map